@@ -6,7 +6,7 @@ require Exporter;
 *import = \&Exporter::import;
 require DynaLoader;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 #$VERSION = eval $VERSION;
 
 DynaLoader::bootstrap Math::MPFI $VERSION;
@@ -111,6 +111,7 @@ TRmpfi_out_str TRmpfi_inp_str
 Rmpfi_get_version Rmpfi_const_catalan Rmpfi_cbrt Rmpfi_hypot
 Rmpfi_sec Rmpfi_csc Rmpfi_cot Rmpfi_sech Rmpfi_csch Rmpfi_coth
 Rmpfi_atan2 Rmpfi_urandom
+Rmpfi_get_NV Rmpfi_set_NV
 );
 
 %Math::MPFI::EXPORT_TAGS =(mpfi => [qw(
@@ -165,11 +166,15 @@ TRmpfi_out_str TRmpfi_inp_str
 Rmpfi_get_version Rmpfi_const_catalan Rmpfi_cbrt Rmpfi_hypot
 Rmpfi_sec Rmpfi_csc Rmpfi_cot Rmpfi_sech Rmpfi_csch Rmpfi_coth
 Rmpfi_atan2 Rmpfi_urandom
+Rmpfi_get_NV Rmpfi_set_NV
 )]);
 
 *TRmpfi_out_str = \&Rmpfi_out_str;
 *TRmpfi_inp_str = \&Rmpfi_inp_str;
 
+$Math::MPFI::NOK_POK = 0; # Set to 1 to allow warnings in new() and overloaded operations when
+                          # a scalar that has set both NOK (NV) and POK (PV) flags is encountered
+
 sub dl_load_flags {0} # Prevent DynaLoader from complaining and croaking
 
 sub new {
@@ -238,9 +243,8 @@ sub new {
     if($type == _NOK_T) {
       if(@_ ) {die "Too many arguments supplied to new() - expected only one"}
        if(_has_longdouble()) {
-         my $t = Math::MPFR->new();
-         Math::MPFR::Rmpfr_set_ld($t, $arg1, Math::MPFR::Rmpfr_get_default_rounding_mode());
-         @ret = Rmpfi_init_set_fr($t);
+         $ret[0] = Math::MPFI->new();
+         Rmpfi_set_NV($ret[0], $arg1);
        }
        else {@ret = Rmpfi_init_set_d($arg1)}
       return $ret[0];
@@ -250,6 +254,12 @@ sub new {
       if(@_ > 1) {die "Too many arguments supplied to new() - expected no more than two"}
       $base = shift if @_;
       if($base < 0 || $base == 1 || $base > 36) {die "Invalid value for base"}
+      if(_SvNOK($arg1)) {
+        set_nok_pok(nok_pokflag() + 1);
+        if($Math::MPFI::NOK_POK) {
+          warn "Scalar passed to new() is both NV and PV. Using PV (string) value";
+        }
+      }
       @ret = Rmpfi_init_set_str($arg1, $base);
       if($ret[1]) {warn "string supplied to new() contained invalid characters"}
       return $ret[0];
@@ -332,12 +342,13 @@ sub overload_not_equiv {
     return !overload_equiv(@_);
 }
 
-sub overload_spaceship {
-    if(Rmpfi_nan_p($_[0]) || ($_[1] != $_[1])) {return undef}
-    if(overload_equiv(@_)) {return 0}
-    if(overload_gt(@_)) {return 1}
-    if(overload_lt(@_)) {return -1}
-}
+# Moved to XS
+# sub overload_spaceship {
+#    if(Rmpfi_nan_p($_[0]) || ($_[1] != $_[1])) {return undef}
+#    if(overload_equiv(@_)) {return 0}
+#    if(overload_gt(@_)) {return 1}
+#    if(overload_lt(@_)) {return -1}
+#}
 
 sub MPFI_VERSION_MAJOR {return _MPFI_VERSION_MAJOR()}
 sub MPFI_VERSION_MINOR {return _MPFI_VERSION_MINOR()}
@@ -519,13 +530,14 @@ Math::MPFI - perl interface to the MPFI (interval arithmetic) library.
 
     These functions assign new values to already initialized intervals
 
-    $si = Rmpfi_set ($rop, $op);
+    $si = Rmpfi_set    ($rop, $op);
     $si = Rmpfi_set_ui ($rop, $ui);
     $si = Rmpfi_set_si ($rop, $si);
-    $si = Rmpfi_set_d ($rop, $double);
-    $si = Rmpfi_set_z ($rop, $z);   # $z is a Math::GMP
+    $si = Rmpfi_set_d  ($rop, $double);
+    $si = Rmpfi_set_NV ($rop, $NV); # $NV is $Config{nvtype}
+    $si = Rmpfi_set_z  ($rop, $z);  # $z is a Math::GMP
                                     # or Math::GMPz object.
-    $si = Rmpfi_set_q ($rop, $q);   # $q is a Math::GMPq object
+    $si = Rmpfi_set_q  ($rop, $q);  # $q is a Math::GMPq object
     $si = Rmpfi_set_fr ($rop, $fr); # $fr is a Math::MPFR object
      Sets the value of $rop from 2nd arg, rounded outward to the precision
      of $rop. (The value of $op is then contained within $rop.)
@@ -657,6 +669,10 @@ Math::MPFI - perl interface to the MPFI (interval arithmetic) library.
      Converts $op to a double, which is the center of $op rounded to the
      nearest double.
 
+    $NV = Rmpfi_get_NV ($op); # $NV is $Config{nvtype}
+     Converts $op to an NV, which is the center of $op rounded to the
+     nearest NV.
+
     Rmpfi_get_fr ($fr, $op); # $fr is a Math::MPFR object
      Converts $op to a floating-point number, which is the center of $op
      rounded to nearest.
@@ -1127,6 +1143,21 @@ Math::MPFI - perl interface to the MPFI (interval arithmetic) library.
     used by the mpfi library that Math::MPFI uses.
     (The function is not exportable.)
 
+   $iv = Math::MPFI::nok_pokflag(); # not exported
+    Returns the value of the nok_pok flag. This flag is
+    initialized to zero, but incemented by 1 whenever a
+    scalar that is both a float (NOK) and string (POK) is passed
+    to new() or to an overloaded operator. The value of the flag
+    therefore tells us how many times such events occurred . The
+    flag can be reset to 0 by running clear_nok_pok().
+
+   Math::MPFI::set_nok_pok($iv); # not exported
+    Resets the nok_pok flag to the value specified by $iv.
+
+   Math::MPFI::clear_nok_pok(); # not exported
+    Resets the nok_pok flag to 0.(Essentially the same as
+    running Math::MPFI::set_nok_pok(0).)
+
    ##############
 
    ERROR HANDLING
@@ -1167,7 +1198,7 @@ Math::MPFI - perl interface to the MPFI (interval arithmetic) library.
 
     This program is free software; you may redistribute it and/or
     modify it under the same terms as Perl itself.
-    Copyright 2010, 2011, 2014 Sisyphus
+    Copyright 2010, 2011, 2014, 2016 Sisyphus
 
 =head1 AUTHOR
 
